from rest_framework import serializers

from django_burl.conf import settings
from django_burl.models import BriefURL


class BurlSerializerBase(serializers.ModelSerializer):
    class Meta:
        model = BriefURL
        fields = ("burl", "url", "user", "description", "enabled", "created", "updated")

    def validate_burl(self, value):
        if value in settings.BURL_BLACKLIST:
            raise serializers.ValidationError(
                f'burl "{value}" is blacklisted by BURL_BLACKLIST setting'
            )
        return value


class BurlSerializer(BurlSerializerBase):
    user = serializers.IntegerField(required=False, source="user.id")


class BurlUserUUIDSerializer(BurlSerializerBase):
    user = serializers.UUIDField(required=False, source="user.id")



from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.urls import reverse

from rest_framework.authtoken.models import Token
from rest_framework.test import APITestCase

from django_burl.models import BriefURL, BriefURLDomainUser
from django_burl.conf import settings


class BurlApiTestCase(APITestCase):
    @classmethod
    def setUpTestData(cls):
        # update the first site to match the default domain used by the testing client
        Site.objects.filter(id=1).update(domain="testserver", name="testserver")
        cls.site = Site.objects.get(id=1)
        cls.site2 = Site.objects.create(domain="go.to", name="goto")
        cls.url = "https://en.wikipedia.org"
        cls.amy = get_user_model().objects.create_user("amy")
        cls.bender = get_user_model().objects.create_user("bender", is_superuser=True)
        cls.farnsworth = get_user_model().objects.create_user("farnsworth")
        cls.fry = get_user_model().objects.create_user("fry")
        cls.hermes = get_user_model().objects.create_user("hermes")
        cls.kif = get_user_model().objects.create_user("kif")
        cls.leela = get_user_model().objects.create_user("leela")
        cls.zoidberg = get_user_model().objects.create_user("zoidberg")
        Token.objects.create(user=cls.amy)
        Token.objects.create(user=cls.bender)
        Token.objects.create(user=cls.farnsworth)
        Token.objects.create(user=cls.fry)
        Token.objects.create(user=cls.hermes)
        Token.objects.create(user=cls.kif)
        Token.objects.create(user=cls.leela)
        Token.objects.create(user=cls.zoidberg)
        BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.fry, role=BriefURLDomainUser.Role.CREATOR
        )
        BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.amy, role=BriefURLDomainUser.Role.EDITOR
        )
        BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.leela, role=BriefURLDomainUser.Role.ADMIN
        )
        BriefURLDomainUser.objects.create(
            site=cls.site2, user=cls.leela, role=BriefURLDomainUser.Role.CREATOR
        )
        BriefURLDomainUser.objects.create(
            site=cls.site2, user=cls.kif, role=BriefURLDomainUser.Role.EDITOR
        )
        BriefURLDomainUser.objects.create(
            site=cls.site2, user=cls.farnsworth, role=BriefURLDomainUser.Role.ADMIN
        )
        if (
            "rest_framework_simplejwt.authentication.JWTAuthentication"
            in settings.REST_FRAMEWORK["DEFAULT_AUTHENTICATION_CLASSES"]
        ):
            cls.simplejwt = True
        else:
            cls.simplejwt = False

    # for some reason, enabling simplejwt causes unauthorized requests to raise
    # 401 instead of 403; the end result is the same, so test for either case
    def assert_401_or_403(self, status_code):
        if self.simplejwt:
            return self.assertEquals(status_code, 401)
        else:
            return self.assertEquals(status_code, 403)


class BurlCreateTests(BurlApiTestCase):
    def test_create_duplicate_same_domain(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, burl="moz", site=self.site
        )
        data = {"url": "https://www.mozilla.org", "burl": "moz"}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 409)
        redirect.refresh_from_db()
        self.assertEquals(redirect.url, self.url)

    def test_create_duplicate_different_domain(self):
        redirect = BriefURL.objects.create(
            user=self.farnsworth, url=self.url, burl="moz", site=self.site2
        )
        data = {"url": "https://www.mozilla.org", "burl": "moz"}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 201)
        redirect2 = BriefURL.objects.get(burl=response.json()["burl"], site=self.site)
        redirect.refresh_from_db()
        self.assertEquals(redirect.burl, "moz")
        self.assertEquals(redirect.user, self.farnsworth)
        self.assertEquals(redirect.url, self.url)
        self.assertEquals(redirect.site, self.site2)
        self.assertEquals(redirect2.burl, "moz")
        self.assertEquals(redirect2.user, self.fry)
        self.assertEquals(redirect2.url, "https://www.mozilla.org")
        self.assertEquals(redirect2.site, self.site)

    def test_create_random_unauthorized(self):
        data = {"url": "https://en.wikipedia.org", "user": self.kif.id}
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assert_401_or_403(response.status_code)

    def test_create_random_no_domain_user(self):
        data = {"url": self.url, "user": self.zoidberg.id}
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.zoidberg.auth_token.key}"
        )
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 403)

    def test_create_random_site1_burl_as_site2_user(self):
        data = {"url": self.url}
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.farnsworth.auth_token.key}"
        )
        response = self.client.post(
            reverse("burls:burls-list"),
            data,
            format="json",
        )
        self.assertEquals(response.status_code, 403)

    def test_create_random_as_creator(self):
        data = {"url": self.url, "description": "test1"}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 201)
        redirect = BriefURL.objects.get(burl=response.json()["burl"])
        self.assertEquals(redirect.user.id, self.fry.id)
        self.assertEquals(redirect.burl, response.json()["burl"])
        self.assertEquals(redirect.url, "https://en.wikipedia.org")
        self.assertEquals(redirect.description, "test1")
        self.assertTrue(redirect.enabled)

    def test_create_specific_as_creator(self):
        data = {"url": self.url, "description": "test", "burl": "burl_test_1"}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 201)
        redirect = BriefURL.objects.get(burl="burl_test_1")
        self.assertEquals(redirect.user.id, self.fry.id)
        self.assertEquals(redirect.burl, "burl_test_1")
        self.assertEquals(redirect.url, "https://en.wikipedia.org")
        self.assertEquals(redirect.description, "test")
        self.assertTrue(redirect.enabled)

    def test_create_random_for_other_user_as_creator(self):
        data = {"url": "https://en.wikipedia.org", "user": self.amy.id}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 403)
        self.assertEquals(
            str(response.json()["error"]),
            "must be superuser to create burl as another user",
        )

    def test_create_random_for_other_user_as_editor(self):
        data = {"url": "https://en.wikipedia.org", "user": self.fry.id}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 403)
        self.assertEquals(
            str(response.json()["error"]),
            "must be superuser to create burl as another user",
        )

    def test_create_random_for_other_user_as_superuser(self):
        data = {"url": "https://en.wikipedia.org", "user": self.leela.id}
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 201)
        self.assertEquals(str(response.json()["user"]), str(self.leela.id))

    def test_create_random_as_superuser(self):
        data = {
            "url": self.url,
            "description": "test1",
            "burl": "burl_test_1",
        }
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 201)
        self.assertEquals(response.json()["user"], self.bender.id)

    def test_create_blacklisted_as_creator(self):
        data = {"url": self.url, "description": "test1", "burl": "admin"}
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.post(reverse("burls:burls-list"), data, format="json")
        self.assertEquals(response.status_code, 403)


class BurlReadTests(BurlApiTestCase):
    def test_detail_no_domain(self):
        redirect = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.zoidberg.auth_token.key}"
        )
        response = self.client.get(url)
        self.assertEquals(response.status_code, 403)

    def test_detail_owned_by_creator_as_creator(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, description="hello, world!", site=self.site
        )
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(redirect.burl, response.json()["burl"])
        self.assertEquals(str(redirect.user.id), str(response.json()["user"]))
        self.assertEquals(redirect.description, response.json()["description"])
        self.assertEquals(redirect.url, response.json()["url"])

    def test_detail_owned_by_creator_as_editor(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, description="hello, world!", site=self.site
        )
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(str(redirect.user.id), str(response.json()["user"]))

    def test_detail_owned_by_editor_as_superuser(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(str(response.json()["user"]), str(redirect.user.id))

    def test_detail_owned_by_other_user_as_creator(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)

    def test_detail_owned_by_admin_as_editor(self):
        redirect = BriefURL.objects.create(
            user=self.leela, url=self.url, site=self.site
        )
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json()["user"], self.leela.id)

    def test_detail_site2_burl_as_site1_user(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site2)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.get(url, SERVER_NAME=self.site2.domain)
        self.assertEquals(response.status_code, 403)

    def test_detail_unauthorized(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        response = self.client.get(url)
        self.assert_401_or_403(response.status_code)

    def test_list_as_creator(self):
        BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        BriefURL.objects.create(user=self.leela, url=self.url, site=self.site)
        url = reverse("burls:burls-list")
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.json()["results"]), 1)
        self.assertEquals(str(response.json()["results"][0]["user"]), str(self.fry.id))

    def test_list_as_editor(self):
        BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        BriefURL.objects.create(user=self.leela, url=self.url, site=self.site)
        url = reverse("burls:burls-list")
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.json()["results"]), 2)
        users = [str(result["user"]) for result in response.json()["results"]]
        self.assertIn(str(self.fry.id), users)
        self.assertIn(str(self.leela.id), users)

    def test_list_as_superuser(self):
        BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        BriefURL.objects.create(user=self.leela, url=self.url, site=self.site)
        url = reverse("burls:burls-list")
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.json()["results"]), 2)
        users = [str(result["user"]) for result in response.json()["results"]]
        self.assertIn(str(self.fry.id), users)
        self.assertIn(str(self.leela.id), users)

    def test_list_no_domain(self):
        BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        url = reverse("burls:burls-list")
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.zoidberg.auth_token.key}"
        )
        response = self.client.get(url)
        self.assertEquals(response.status_code, 403)

    def test_list_site2_as_site1_user(self):
        BriefURL.objects.create(user=self.leela, url=self.url, site=self.site2)
        BriefURL.objects.create(user=self.kif, url=self.url, site=self.site2)
        url = reverse("burls:burls-list")
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.get(url, SERVER_NAME=self.site2.domain)
        self.assertEquals(response.status_code, 403)

    def test_list_unauthorized(self):
        BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-list")
        response = self.client.get(url)
        self.assert_401_or_403(response.status_code)


class BurlUpdateTests(BurlApiTestCase):
    def test_update_change_user_as_non_superuser(self):
        redirect = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        data = {
            "url": "https://facebook.com",
            "description": "friendface",
            "burl": "zuck",
            "user": self.leela.id,
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.put(url, data, format="json")
        self.assertEquals(response.status_code, 403)

    def test_update_change_user_as_superuser(self):
        burl = BriefURL.objects.create(user=self.leela, url=self.url, site=self.site)
        data = {"url": "https://facebook.com", "user": self.fry.id}
        url = reverse("burls:burls-detail", kwargs={"burl": burl.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        burl.refresh_from_db()
        self.assertEquals(burl.url, "https://facebook.com")
        self.assertEquals(burl.user, self.fry)

    def test_update_patch_by_admin_as_creator(self):
        redirect = BriefURL.objects.create(
            user=self.leela, url=self.url, site=self.site, description="leela's link"
        )
        data = {"url": "https://www.mozilla.org"}
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 404)

    def test_update_patch_by_admin_as_editor(self):
        redirect = BriefURL.objects.create(
            user=self.leela, url=self.url, site=self.site, description="leela's link"
        )
        start_burl = redirect.burl
        data = {"url": "https://www.mozilla.org"}
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        redirect.refresh_from_db()
        self.assertEquals(redirect.burl, start_burl)
        self.assertEquals(redirect.description, "leela's link")
        self.assertEquals(redirect.url, "https://www.mozilla.org")
        self.assertEquals(redirect.user, self.leela)

    def test_update_patch_by_creator_as_creator(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, site=self.site, description="fry's link"
        )
        start_burl = redirect.burl
        data = {
            "url": "https://twitter.com",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        redirect.refresh_from_db()
        self.assertEquals(redirect.burl, start_burl)
        self.assertEquals(redirect.description, "fry's link")
        self.assertEquals(redirect.url, "https://twitter.com")
        self.assertEquals(redirect.user, self.fry)

    def test_update_patch_by_creator_as_editor(self):
        burl = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        data = {"url": "https://facebook.com"}
        url = reverse("burls:burls-detail", kwargs={"burl": burl.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        burl.refresh_from_db()
        self.assertEquals(burl.url, "https://facebook.com")

    def test_update_patch_no_domain(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site)
        data = {
            "description": "friendface",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.zoidberg.auth_token.key}"
        )
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 403)

    def test_update_patch_unauthorized(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site)
        data = {
            "description": "friendface",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        response = self.client.patch(url, data, format="json")
        self.assert_401_or_403(response.status_code)

    def test_update_put_by_creator_as_creator(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, site=self.site, description="fry's link"
        )
        data = {
            "url": "https://twitter.com",
            "description": "twitter",
            "burl": "twitter",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        response = self.client.put(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        redirect.refresh_from_db()
        self.assertEquals(redirect.burl, "twitter")
        self.assertEquals(redirect.description, "twitter")
        self.assertEquals(redirect.url, "https://twitter.com")
        self.assertEquals(redirect.user, self.fry)

    def test_update_put_by_creator_as_editor(self):
        redirect = BriefURL.objects.create(
            user=self.fry, url=self.url, site=self.site, description="fry's link"
        )
        data = {
            "url": "https://www.mozilla.org",
            "description": "mozilla",
            "burl": "moz",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.put(url, data, format="json")
        self.assertEquals(response.status_code, 200)
        redirect.refresh_from_db()
        self.assertEquals(redirect.burl, "moz")
        self.assertEquals(redirect.description, "mozilla")
        self.assertEquals(redirect.url, "https://www.mozilla.org")
        self.assertEquals(redirect.user, self.fry)

    def test_update_put_no_domain(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site)
        data = {
            "url": "https://www.mozilla.org",
            "description": "friendface",
            "burl": "moz",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.zoidberg.auth_token.key}"
        )
        response = self.client.put(url, data, format="json")
        self.assertEquals(response.status_code, 403)

    def test_update_put_unauthorized(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site)
        data = {
            "url": "https://facebook.com",
            "description": "friendface",
            "burl": "zuck",
        }
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        response = self.client.put(url, data, format="json")
        self.assert_401_or_403(response.status_code)

    def test_update_site2_burl_as_site1_editor(self):
        redirect = BriefURL.objects.create(
            user=self.farnsworth, url=self.url, site=self.site2, description="old man"
        )
        data = {"url": "https://facebook.com", "description": "friendface"}
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.patch(url, data, format="json")
        self.assertEquals(response.status_code, 404)


class BurlDeleteTests(BurlApiTestCase):
    def test_delete_by_admin_as_superuser(self):
        redirect = BriefURL.objects.create(
            user=self.leela, url=self.url, site=self.site
        )
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.bender.auth_token.key}"
        )
        delete = self.client.delete(url)
        self.assertEquals(delete.status_code, 204)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 0)

    def test_delete_by_creator_as_creator(self):
        redirect = BriefURL.objects.create(user=self.kif, url=self.url, site=self.site2)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.kif.auth_token.key}")
        response = self.client.delete(url, SERVER_NAME=self.site2.domain)
        self.assertEquals(response.status_code, 204)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 0)

    def test_delete_by_editor_as_creator(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.fry.auth_token.key}")
        delete = self.client.delete(url)
        self.assertEquals(delete.status_code, 404)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 1)

    def test_delete_by_creator_as_admin(self):
        redirect = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.leela.auth_token.key}")
        response = self.client.delete(url)
        self.assertEquals(response.status_code, 204)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 0)

    def test_delete_by_creator_as_editor(self):
        redirect = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {self.amy.auth_token.key}")
        response = self.client.delete(url)
        self.assertEquals(response.status_code, 403)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 1)

    def test_delete_no_domain(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.farnsworth.auth_token.key}"
        )
        delete = self.client.delete(url)
        self.assertEquals(delete.status_code, 403)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 1)

    def test_delete_site1_burl_as_site2_admin(self):
        redirect = BriefURL.objects.create(user=self.fry, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        self.client.credentials(
            HTTP_AUTHORIZATION=f"Token {self.farnsworth.auth_token.key}"
        )
        delete = self.client.delete(url)
        self.assertEquals(delete.status_code, 403)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 1)

    def test_delete_unauthorized(self):
        redirect = BriefURL.objects.create(user=self.amy, url=self.url, site=self.site)
        url = reverse("burls:burls-detail", kwargs={"burl": redirect.burl})
        response = self.client.delete(url)
        self.assert_401_or_403(response.status_code)
        self.assertEquals(len(BriefURL.objects.filter(burl=redirect.burl)), 1)



import secrets

from hashids import Hashids

from django_burl.conf import settings


def make_burl(ceiling=1000):
    """
    Generates a non-unique burl short url

    :param ceiling: maximum integer for generating random number
    :return: burl short url
    :rtype: str
    """
    salt = secrets.token_hex(4)
    hasher = Hashids(salt=salt, alphabet=settings.HASHID_ALPHABET)
    random = secrets.randbelow(abs(ceiling) + 1000)
    return hasher.encode(random)



# Generated by Django 4.0.2 on 2022-02-25 21:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0007_briefurldefaultredirect'),
    ]

    operations = [
        migrations.AlterField(
            model_name='briefurldefaultredirect',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site', unique=True, verbose_name='site'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import TestCase

from django_burl import utils
from django_burl.models import BriefURL
from django_burl.admin import BriefURLSuperForm


class BriefURLAdminFormTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        zoid = get_user_model().objects.create_user("zoidberg")
        site = Site.objects.first()
        cls.user = zoid
        cls.site = site
        cls.redirect = BriefURL.objects.create(
            user=zoid, url="https://google.com", site=site
        )

    def test_init(self):
        BriefURLSuperForm(self.redirect)

    def test_blacklisted_burl(self):
        redirect = self.redirect.__dict__
        redirect["burl"] = "admin"
        form = BriefURLSuperForm(redirect)
        self.assertTrue("burl" in form.errors)
        self.assertFalse(form.is_valid())

    def test_valid_burl(self):
        redirect = self.redirect.__dict__
        redirect["burl"] = utils.make_burl(1000)
        redirect["url"] = "https://google.com"
        redirect["user"] = self.user.id
        redirect["site"] = self.site
        form = BriefURLSuperForm(redirect)
        self.assertTrue(form.is_valid())



import uuid

from django.contrib.auth import get_user_model
from django.shortcuts import get_object_or_404
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet
from rest_framework import permissions

from django_burl.models import BriefURL
from django_burl.api.v1.serializers import BurlSerializer, BurlUserUUIDSerializer


class IsOwner(permissions.BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.user.is_superuser:
            return True
        return obj.user == request.user


class BriefURLViewSet(ModelViewSet):
    permission_classes = (IsOwner, IsAuthenticated)
    filterset_fields = {
        "enabled": ["exact"],
        "description": ["exact", "icontains"],
        "created": ["exact", "lt", "gt", "lte", "gte"],
        "updated": ["exact", "lt", "gt", "lte", "gte"],
        "url": ["exact", "icontains"],
        "burl": ["exact", "icontains"],
    }
    lookup_field = "burl"

    def get_queryset(self):
        if self.request.user.is_superuser:
            return BriefURL.objects.order_by("-created")
        elif self.request.user.is_authenticated:
            return BriefURL.objects.filter(user=self.request.user).order_by("-created")
        else:
            return []

    def get_serializer_class(self):
        if type(self.request.user.id) == uuid.UUID:
            return BurlUserUUIDSerializer
        return BurlSerializer

    def perform_create(self, serializer):
        if (
            self.request.user.is_superuser
            and "user" in serializer._validated_data.keys()
        ):
            user = get_object_or_404(
                get_user_model(), id=serializer._validated_data["user"]["id"]
            )
            serializer.save(user=user)
        else:
            serializer.save(user=self.request.user)

    def perform_update(self, serializer):
        if (
            self.request.user.is_superuser
            and "user" in serializer._validated_data.keys()
        ):
            user = get_object_or_404(
                get_user_model(), id=serializer._validated_data["user"]["id"]
            )
            serializer.save(user=user)
        else:
            serializer.save(user=self.request.user)



# Generated by Django 4.0.2 on 2022-02-25 17:47

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_burl', '0005_briefurldomainuser'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='briefurldomainuser',
            unique_together={('site', 'user')},
        ),
    ]



import uuid

from django.contrib.auth import get_user_model
from django.db import IntegrityError
from django.shortcuts import get_object_or_404
from rest_framework.exceptions import PermissionDenied, APIException
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet
from rest_framework import permissions

from django_burl.models import BriefURL, get_domain_user
from django_burl.api.v2.serializers import BurlSerializer, BurlUserUUIDSerializer


class ConflictException(APIException):
    status_code = 409
    default_detail = "your request conflicts with existing data"
    default_code = "resource_conflict"


class IsOwner(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.user.is_superuser:
            return True
        if domain_user := get_domain_user(request.user, request.site):
            if domain_user.is_creator:
                return True
        return False

    def has_object_permission(self, request, view, obj):
        if request.user.is_superuser:
            return True
        if domain_user := get_domain_user(request.user, request.site):
            if (
                domain_user.is_creator
                and obj.user == request.user
                and obj.site == request.site
            ):
                return True
            elif domain_user.is_editor and obj.site == request.site:
                return True
        return False


class BriefURLViewSet(ModelViewSet):
    permission_classes = (IsOwner, IsAuthenticated)
    filterset_fields = {
        "enabled": ["exact"],
        "description": ["exact", "icontains"],
        "created": ["exact", "lt", "gt", "lte", "gte"],
        "updated": ["exact", "lt", "gt", "lte", "gte"],
        "url": ["exact", "icontains"],
        "burl": ["exact", "icontains"],
        "user": ["exact"],
    }
    lookup_field = "burl"

    def get_queryset(self):
        if self.request.user.is_superuser:
            return BriefURL.objects.filter(site=self.request.site).order_by("-created")
        elif self.request.user.is_authenticated:
            if domain_user := get_domain_user(self.request.user, self.request.site):
                if domain_user.is_editor:
                    return BriefURL.objects.filter(site=self.request.site).order_by(
                        "-created"
                    )
                elif domain_user.is_creator:
                    return BriefURL.objects.filter(
                        user=self.request.user, site=self.request.site
                    ).order_by("-created")
        return BriefURL.objects.none()

    def get_serializer_class(self):
        if type(self.request.user.id) == uuid.UUID:
            return BurlUserUUIDSerializer
        return BurlSerializer

    def perform_create(self, serializer):
        if "user" in serializer.validated_data.keys():
            if self.request.user.is_superuser:
                user = get_object_or_404(
                    get_user_model(), id=serializer.validated_data["user"]["id"]
                )
                return self._save_with_integrity(serializer, user)
            else:
                raise PermissionDenied(
                    {"error": "must be superuser to create burl as another user"}
                )
        else:
            return self._save_with_integrity(serializer, self.request.user)

    def perform_destroy(self, instance):
        if self.request.user.is_superuser or self.request.user == instance.user:
            return instance.delete()
        else:
            if domain_user := get_domain_user(self.request.user, self.request.site):
                if domain_user.is_admin:
                    return instance.delete()
        raise PermissionDenied(
            {"error": "must be superuser or domain admin to delete other user's burl"}
        )

    def perform_update(self, serializer):
        if "user" in serializer.validated_data.keys():
            if serializer.validated_data["user"]["id"] != serializer.instance.user.id:
                if self.request.user.is_superuser:
                    user = get_object_or_404(
                        get_user_model(), id=serializer.validated_data["user"]["id"]
                    )
                    return self._save_with_integrity(serializer, user)
                else:
                    raise PermissionDenied(
                        {"error": "must be superuser to change burl user"}
                    )
            else:
                return self._save_with_integrity(serializer, serializer.instance.user)
        else:
            return self._save_with_integrity(serializer, serializer.instance.user)

    def _save_with_integrity(self, serializer, user):
        try:
            return serializer.save(user=user, site=self.request.site)
        except IntegrityError:
            raise ConflictException(
                {
                    "error": "burl already exists",
                    "burl": serializer.validated_data["burl"],
                }
            )



from django.test import TestCase

from django_burl import utils


class MakeBurlTest(TestCase):
    def test_make_burl_return_string(self):
        burl = utils.make_burl()
        self.assertIsInstance(burl, str)



from rest_framework import serializers
from rest_framework.exceptions import PermissionDenied

from django_burl.conf import settings
from django_burl.models import BriefURL


class BurlSerializerBase(serializers.ModelSerializer):
    class Meta:
        model = BriefURL
        fields = (
            "burl",
            "url",
            "user",
            "description",
            "enabled",
            "created",
            "updated",
        )

    def validate_burl(self, value):
        if value in settings.BURL_BLACKLIST:
            raise PermissionDenied(
                {"error": "blacklisted by BURL_BLACKLIST setting", "burl": value}
            )
        return value


class BurlSerializer(BurlSerializerBase):
    user = serializers.IntegerField(required=False, source="user.id")


class BurlUserUUIDSerializer(BurlSerializerBase):
    user = serializers.UUIDField(required=False, source="user.id")



from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.reverse import reverse


@api_view(["GET"])
def api_root(request, format=None):
    return Response(
        {
            "burls": reverse("api_v2:burls:burls-list", request=request, format=format),
        }
    )



# Generated by Django 4.0.2 on 2022-02-24 22:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_burl', '0003_briefurl_site_alter_briefurl_burl_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='briefurl',
            name='burl_site_enabled',
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['burl', 'site', 'enabled'], name='django_burl_briefurl_brl_st_on'),
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['user', 'site'], name='django_burl_briefurl_user_site'),
        ),
    ]



default_app_config = "django_burl.apps.DjangoBurlConfig"

try:
    from django.conf import settings
    from django.core.exceptions import ImproperlyConfigured

    for appname in [
        "django.contrib.sites",
        "django_filters",
        "django_burl",
        "rest_framework",
    ]:
        if appname not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured(
                f"{appname} must be present in the INSTALLED_APPS setting"
            )

    for ware in ["django.contrib.sites.middleware.CurrentSiteMiddleware"]:
        if ware not in settings.MIDDLEWARE:
            raise ImproperlyConfigured(
                f"{ware} must be present in the MIDDLEWARE setting"
            )

except AttributeError as ex:
    raise ImproperlyConfigured(f"{ex} - is django configured properly?")

except ImportError:
    import logging

    logger = logging.getLogger(__name__)
    logger.warning("django does not appear to be installed")



from django.urls import include, path

from django_burl import views
from django_burl.api.v1 import urls as api_v1
from django_burl.api.v2 import urls as api_v2

app_name = "django_burl"

urlpatterns = [
    path("<str:burl>/", views.get_redirect, name="redirect"),
    path("api/v1/", include(api_v1)),
    path("api/v2/", include(api_v2)),
]



# Generated by Django 4.0.2 on 2022-02-25 17:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0004_remove_briefurl_burl_site_enabled_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='BriefURLDomainUser',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('role', models.CharField(choices=[('creator', 'Creator'), ('editor', 'Editor'), ('admin', 'Admin')], default='creator', max_length=16)),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.2.27 on 2022-02-17 21:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_burl", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="briefurl",
            name="id",
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 4.0.2 on 2022-02-24 20:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0002_auto_20220217_1532'),
    ]

    operations = [
        migrations.AddField(
            model_name='briefurl',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='sites.site'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='briefurl',
            name='burl',
            field=models.CharField(blank=True, db_index=True, max_length=2048, verbose_name='Brief URL'),
        ),
        migrations.AlterField(
            model_name='briefurl',
            name='enabled',
            field=models.BooleanField(db_index=True, default=True),
        ),
        migrations.AlterUniqueTogether(
            name='briefurl',
            unique_together={('burl', 'site')},
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['burl', 'site', 'enabled'], name='burl_site_enabled'),
        ),
    ]



import logging
from django.db import connection, connections
from django.db.models.query import QuerySet

from django_burl.conf import settings

logger = logging.getLogger(__name__)


class RoughCountQuerySet(QuerySet):
    def rough_count(self):
        if connection.vendor == "postgresql":
            cursor = connections[self.db].cursor()
            try:
                cursor.execute(
                    f"SELECT reltuples FROM pg_class WHERE relname='{self.query.model._meta.db_table}';"
                )
                rough = int(cursor.fetchone()[0])
                if rough > settings.ROUGH_COUNT_MIN:
                    return rough
                return super().count()
            except Exception as ex:
                logger.error(f"error getting rough count from postgres: {ex}")
                return super().count()
        return super().count()



from datetime import datetime

from django.contrib.sites.models import Site
from django.db import connection, IntegrityError
from django.test import TestCase
from django.core.exceptions import ValidationError
from django.contrib.auth import get_user_model

from django_burl.conf import settings
from django_burl.models import BriefURL, BriefURLDomainUser


class BriefURLModelTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = get_user_model().objects.create_user("leela")
        cls.url = "https://google.com"
        cls.description = "test redirect"
        cls.site = Site.objects.first()

    def test_create_redirect_random_burl(self):
        redirect = BriefURL.objects.create(
            url=self.url, description=self.description, user=self.user
        )
        self.assertIsInstance(redirect, BriefURL)
        self.assertEquals(redirect.url, "https://google.com")
        self.assertIsInstance(redirect.burl, str)
        self.assertEquals(redirect.description, "test redirect")
        self.assertIsInstance(redirect.created, datetime)
        self.assertIsInstance(redirect.created, datetime)
        self.assertTrue(redirect.enabled)

    def test_create_redirect_custom_burl(self):
        redirect = BriefURL.objects.create(
            url=self.url, description=self.description, user=self.user, burl="google"
        )
        self.assertIsInstance(redirect, BriefURL)
        self.assertEquals(redirect.url, "https://google.com")
        self.assertEquals(redirect.burl, "google")
        self.assertEquals(redirect.description, "test redirect")
        self.assertIsInstance(redirect.created, datetime)
        self.assertIsInstance(redirect.created, datetime)
        self.assertTrue(redirect.enabled)

    def test_create_redirect_custom_blacklisted_burl(self):
        with self.assertRaises(ValidationError):
            BriefURL.objects.create(user=self.user, url=self.url, burl="admin")

    def test_create_redirect_custom_duplicate_burl(self):
        BriefURL.objects.create(
            url=self.url, user=self.user, burl="test_duplicate", site=self.site
        )
        with self.assertRaises(IntegrityError):
            BriefURL.objects.create(
                url=self.url, user=self.user, burl="test_duplicate", site=self.site
            )


class BriefURLManagerTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        if connection.vendor == "postgresql":
            cls.min = settings.ROUGH_COUNT_MIN - 100
            cls.max = settings.ROUGH_COUNT_MIN * 2
            cls.user = get_user_model().objects.create_user("hermes")
            cls.url = "https://google.com"
            cls.site = Site.objects.first()
            count = 0
            while count < cls.min:
                BriefURL.objects.create(user=cls.user, url=cls.url)
                count += 1

    def test_rough_count_under_min(self):
        if connection.vendor == "postgresql":
            self.assertIsInstance(BriefURL.objects.count(), int)
            self.assertIsInstance(BriefURL.objects.rough_count(), int)
            self.assertEquals(BriefURL.objects.count(), BriefURL.objects.rough_count())

    def test_rough_count_over_min(self):
        if connection.vendor == "postgresql":
            count = BriefURL.objects.count()
            while self.max - count > 0:
                BriefURL.objects.create(user=self.user, url=self.url, site=self.site)
                count += 1
            count = BriefURL.objects.count()
            rough_count = BriefURL.objects.rough_count()
            self.assertIsInstance(count, int)
            self.assertIsInstance(rough_count, int)
            self.assertTrue(abs(count - rough_count) < count * 0.5)


class BriefURLDomainUserTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.fry = get_user_model().objects.create_user("fry")
        cls.amy = get_user_model().objects.create_user("amy")
        cls.leela = get_user_model().objects.create_user("leela")
        cls.url = "https://google.com"
        cls.description = "test redirect"
        cls.site = Site.objects.first()
        cls.fry_user_domain = BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.fry, role=BriefURLDomainUser.Role.CREATOR
        )
        cls.amy_user_domain = BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.amy, role=BriefURLDomainUser.Role.EDITOR
        )
        cls.leela_user_domain = BriefURLDomainUser.objects.create(
            site=cls.site, user=cls.leela, role=BriefURLDomainUser.Role.ADMIN
        )

    def test_is_creator(self):
        self.assertTrue(self.fry_user_domain.is_creator)
        self.assertTrue(self.amy_user_domain.is_creator)
        self.assertTrue(self.leela_user_domain.is_creator)

    def test_is_editor(self):
        self.assertFalse(self.fry_user_domain.is_editor)
        self.assertTrue(self.amy_user_domain.is_editor)
        self.assertTrue(self.leela_user_domain.is_editor)

    def test_is_admin(self):
        self.assertFalse(self.fry_user_domain.is_admin)
        self.assertFalse(self.amy_user_domain.is_admin)
        self.assertTrue(self.leela_user_domain.is_admin)



# Generated by Django 2.0.5 on 2019-02-21 22:47

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="BriefURL",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "url",
                    models.URLField(db_index=True, max_length=2048, verbose_name="URL"),
                ),
                (
                    "burl",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        max_length=2048,
                        unique=True,
                        verbose_name="Brief URL",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        blank=True,
                        help_text="description of the destination URL",
                        max_length=255,
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



import logging

from django.contrib.sites.shortcuts import get_current_site
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import get_object_or_404

from django_burl.models import BriefURL


logger = logging.getLogger(__name__)


def get_redirect(request, burl):
    site = get_current_site(request)
    redirect = get_object_or_404(BriefURL, burl=burl, site=site, enabled=True)
    return HttpResponseRedirect(redirect.url)



from rest_framework import routers
from django.urls import include, path

from django_burl.api.v1 import views, viewsets

router = routers.SimpleRouter()
router.register(r"redirects", viewsets.BriefURLViewSet, basename="redirect")


urlpatterns = [
    path("", views.api_root, name="redirects-root"),
    path("", include((router.urls, "redirects"))),
]



from django.contrib.sites.models import Site
from django.test import TestCase
from django.contrib.auth import get_user_model
from django.urls import reverse

from django_burl.models import BriefURL


class BriefURLViewTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        Site.objects.filter(id=1).update(domain="testserver")
        site = Site.objects.first()
        cls.user = get_user_model().objects.create_user("fry")
        cls.redirect = BriefURL.objects.create(
            user=cls.user, url="https://twitter.com", site=site
        )
        cls.redirect_disabled = BriefURL.objects.create(
            user=cls.user, url="https://google.com", enabled=False, site=site
        )

    def test_redirect(self):
        url = reverse("redirect", kwargs={"burl": self.redirect.burl})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response.headers["location"], "https://twitter.com")

    def test_redirect_not_found(self):
        url = reverse("redirect", kwargs={"burl": "asdf1234"})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)

    def test_redirect_disabled(self):
        url = reverse("redirect", kwargs={"burl": self.redirect_disabled.burl})
        response = self.client.get(url)
        self.assertEquals(response.status_code, 404)



from rest_framework import routers
from django.urls import include, path

from django_burl.api.v2 import views
from django_burl.api.v2 import viewsets

router = routers.SimpleRouter()
router.register(r"burls", viewsets.BriefURLViewSet, basename="burls")


urlpatterns = [
    path("", views.api_root, name="burls-root"),
    path("", include((router.urls, "burls"))),
]



from django.conf import settings

if not hasattr(settings, "BURL_BLACKLIST"):
    settings.BURL_BLACKLIST = ["admin", "api", "static", "media"]

if not hasattr(settings, "ROUGH_COUNT_MIN"):
    settings.ROUGH_COUNT_MIN = 1000

if not hasattr(settings, "HASHID_ALPHABET"):
    settings.HASHID_ALPHABET = (
        "abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ0123456789"
    )



from django.contrib import admin
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.utils.translation import gettext as _
from django.core.exceptions import ValidationError, ObjectDoesNotExist

from django_burl.conf import settings
from django_burl.models import BriefURL, BriefURLDomainUser, BriefURLDefaultRedirect


class BriefURLMixin:
    class Meta:
        model = BriefURL
        fields = "__all__"

    def clean_burl(self):
        burl = self.cleaned_data["burl"]
        if burl in settings.BURL_BLACKLIST:
            raise ValidationError(
                _("The specified brief URL is blacklisted and cannot be used!")
            )
        return burl


class BriefURLSuperForm(forms.ModelForm, BriefURLMixin):
    user = forms.ModelChoiceField(get_user_model().objects.all(), required=True)
    site = forms.ModelChoiceField(Site.objects.all(), required=True)


class BriefUrlEditorForm(forms.ModelForm, BriefURLMixin):
    user = forms.ModelChoiceField(
        get_user_model().objects.all(), disabled=True, required=True
    )
    site = forms.ModelChoiceField(Site.objects.all(), disabled=True, required=True)


class BriefURLAdmin(admin.ModelAdmin):
    def get_form(self, request, obj=None, **kwargs):
        if request.user.is_superuser:
            form = BriefURLSuperForm
        else:
            form = BriefUrlEditorForm
        form.base_fields["user"].initial = request.user
        form.base_fields["site"].initial = request.site
        return form

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super().get_queryset(request)
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_editor:
                return super().get_queryset(request).filter(site=request.site)
            elif domain_user.is_creator:
                return (
                    super()
                    .get_queryset(request)
                    .filter(site=request.site, user=request.user)
                )
            else:
                return BriefURL.objects.none()
        except ObjectDoesNotExist:
            return BriefURL.objects.none()

    def has_add_permission(self, request, obj=None):
        if request.user.is_superuser:
            return True
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_creator:
                return True
        except ObjectDoesNotExist:
            return False
        return False

    def has_change_permission(self, request, obj=None):
        return self._domain_user_owner(request, obj)

    def has_delete_permission(self, request, obj=None):
        if request.user.is_superuser:
            return True
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_admin:
                return True
            if domain_user.is_creator:
                if obj and obj.user == request.user:
                    return True
                elif not obj:
                    return True
        except ObjectDoesNotExist:
            return False
        return False

    def has_view_permission(self, request, obj=None):
        return self._domain_user_owner(request, obj)

    def _domain_user_owner(self, request, obj):
        if request.user.is_superuser:
            return True
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_editor:
                return True
            if domain_user.is_creator:
                if obj and obj.user == request.user:
                    return True
                elif not obj:
                    return True
        except ObjectDoesNotExist:
            return False
        return False


class BriefURLDefaultRedirectMixin:
    class Meta:
        model = BriefURLDefaultRedirect
        fields = "__all__"


class BriefURLDefaultRedirectSuperForm(forms.ModelForm, BriefURLDefaultRedirectMixin):
    pass


class BriefURLDefaultRedirectEditorForm(forms.ModelForm, BriefURLDefaultRedirectMixin):
    site = forms.ModelChoiceField(Site.objects.all(), disabled=True)


class BriefURLDefaultRedirectAdmin(admin.ModelAdmin):
    def get_form(self, request, obj=None, **kwargs):
        if request.user.is_superuser:
            form = BriefURLDefaultRedirectSuperForm
        else:
            form = BriefURLDefaultRedirectEditorForm
        form.base_fields["site"].initial = request.site
        return form

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super().get_queryset(request)
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_admin:
                return super().get_queryset(request).filter(site=request.site)
            else:
                return super().get_queryset(request).none()
        except ObjectDoesNotExist:
            return super().get_queryset(request).none()

    def has_view_permission(self, request, *args, **kwargs):
        return self._super_or_domain_admin(request)

    def has_change_permission(self, request, *args, **kwargs):
        return self._super_or_domain_admin(request)

    def has_add_permission(self, request, *args, **kwargs):
        return self._super_or_domain_admin(request)

    def has_delete_permission(self, request, *args, **kwargs):
        return self._super_or_domain_admin(request)

    def _super_or_domain_admin(self, request):
        if request.user.is_superuser:
            return True
        try:
            domain_user = BriefURLDomainUser.objects.get(
                user=request.user, site=request.site
            )
            if domain_user.is_admin:
                return True
        except ObjectDoesNotExist:
            return False
        return False


admin.site.register(BriefURL, BriefURLAdmin)
admin.site.register(BriefURLDomainUser)
admin.site.register(BriefURLDefaultRedirect, BriefURLDefaultRedirectAdmin)



import re
import time

import requests

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

from django_burl.models import BriefURL


class Command(BaseCommand):
    help = "imports URLs from the PLOS solr API"

    def add_arguments(self, parser):
        parser.add_argument("user", type=str)
        parser.add_argument("start_date", type=str)
        parser.add_argument("end_date", type=str)
        parser.add_argument("--rows", "-r", type=str, default=500)

    def handle(self, *args, **options):
        user = get_user_model().objects.get(username=options["user"])
        start_date, end_date, rows = (
            options["start_date"],
            options["end_date"],
            options["rows"],
        )
        response = requests.get(_make_url(start_date, end_date, 0, rows))
        total = response.json().get("response", {}).get("numFound", 0)
        docs = response.json().get("response", {}).get("docs", [])
        _save_results(docs, user)
        saved = len(docs)
        while saved < total:
            response = requests.get(_make_url(start_date, end_date, saved, rows))
            docs = response.json().get("response", {}).get("docs", [])
            _save_results(docs, user)
            saved += len(docs)
            time.sleep(0.5)
        print(saved)


def _make_url(start_date, end_date, row_start, rows):
    qstring = f"publication_date:[{start_date} TO {end_date}] AND abstract:[* TO *]"
    qstring = "&".join([qstring, "fl=id,title"])
    qstring = "&".join([qstring, f"&rows={rows}&start={row_start}"])
    qstring = re.sub(r"\s+", "%20", qstring)
    return f"https://api.plos.org/search?q={qstring}"


def _save_results(results, user):
    for result in results:
        redirect = BriefURL.objects.create(
            url=_get_url(result["id"]), description=result["title"], user=user
        )
        print(f"/{redirect.burl} -> {redirect.url} ({redirect.description})")
        time.sleep(0.5)


def _get_url(doi):
    response = requests.head(f"https://dx.plos.org/{doi}")
    if response.status_code != 301:
        return f"https://dx.plos.org/{doi}"
    return response.headers["Location"]



# Generated by Django 4.0.2 on 2022-02-25 21:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0006_alter_briefurldomainuser_unique_together'),
    ]

    operations = [
        migrations.CreateModel(
            name='BriefURLDefaultRedirect',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('url', models.URLField(verbose_name='redirect to')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='site')),
            ],
        ),
    ]



from django.contrib.sites.models import Site
from django.db.models.manager import BaseManager
from django.db.utils import IntegrityError
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.contrib.auth import get_user_model
from django.db import models, transaction

from django_burl import utils
from django_burl.conf import settings
from django_burl.database import RoughCountQuerySet


class BriefURLManager(BaseManager.from_queryset(RoughCountQuerySet)):
    pass


class BriefURL(models.Model):
    id = models.BigAutoField(primary_key=True)
    url = models.URLField(max_length=2048, db_index=True, verbose_name="URL")
    burl = models.CharField(
        max_length=2048,
        blank=True,
        db_index=True,
        verbose_name="Brief URL",
    )
    description = models.CharField(
        max_length=255, blank=True, help_text="description of the destination URL"
    )
    user = models.ForeignKey(get_user_model(), on_delete=models.PROTECT)
    site = models.ForeignKey(Site, on_delete=models.PROTECT)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    enabled = models.BooleanField(default=True, db_index=True)
    objects = BriefURLManager()

    def __str__(self):
        return f"{self.site.domain}/{self.burl} → {self.url}"

    class Meta:
        unique_together = [["burl", "site"]]
        indexes = [
            models.Index(
                fields=["burl", "site", "enabled"],
                name="django_burl_briefurl_brl_st_on",
            ),
            models.Index(
                fields=["user", "site"], name="django_burl_briefurl_user_site"
            ),
        ]

    def _set_burl(self):
        if self.burl:
            self.random = False
            if self.burl in settings.BURL_BLACKLIST:
                raise ValidationError(
                    f'burl "{self.burl}" is blacklisted by BURL_BLACKLIST setting'
                )
        else:
            self.random = True
            self.burl = utils.make_burl(ceiling=BriefURL.objects.rough_count())
            if self.burl in settings.BURL_BLACKLIST:
                self.burl = None
                return self._set_burl()

    def save(self, *args, **kwargs):
        self._set_burl()
        try:
            with transaction.atomic():
                return super().save(*args, **kwargs)
        except IntegrityError as ex:
            if "unique constraint" in repr(ex):
                if self.random:
                    self.burl = None
                    return self.save(*args, **kwargs)
                else:
                    raise ex


class BriefURLDomainUser(models.Model):
    class Role(models.TextChoices):
        CREATOR = "creator"
        EDITOR = "editor"
        ADMIN = "admin"

    id = models.BigAutoField(primary_key=True)
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    site = models.ForeignKey(Site, on_delete=models.CASCADE)
    role = models.CharField(max_length=16, choices=Role.choices, default=Role.CREATOR)

    def __str__(self):
        return f"@{self.user.username}: {self.role} on {self.site.domain}"

    class Meta:
        unique_together = [["site", "user"]]

    @property
    def is_creator(self):
        return self.role in [self.Role.CREATOR, self.Role.EDITOR, self.Role.ADMIN]

    @property
    def is_editor(self):
        return self.role in [self.Role.EDITOR, self.Role.ADMIN]

    @property
    def is_admin(self):
        return self.role == self.Role.ADMIN


def get_domain_user(user, site):
    try:
        return BriefURLDomainUser.objects.get(user=user, site=site)
    except (TypeError, ObjectDoesNotExist, ValidationError):
        return None


class BriefURLDefaultRedirect(models.Model):
    id = models.AutoField(primary_key=True)
    site = models.OneToOneField(
        Site, on_delete=models.CASCADE, verbose_name="site", unique=True
    )
    url = models.URLField(verbose_name="redirect to")

    def __str__(self):
        return f"{self.site.domain} -> {self.url}"



from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.reverse import reverse


@api_view(["GET"])
def api_root(request, format=None):
    return Response(
        {
            "redirects": reverse(
                "api_v1:redirects:redirect-list", request=request, format=format
            ),
        }
    )



import os


def get_env(name, default=None):
    env = os.environ.get(name)
    return env if env else default


DEBUG = False
ALLOWED_HOSTS = ["*"]

SETTINGS_MODULE = os.path.dirname(os.path.abspath(__file__))
MODULE_ROOT = os.path.dirname(SETTINGS_MODULE)
BASE_DIR = MODULE_ROOT
PROJECT_ROOT = os.path.dirname(MODULE_ROOT)

HOME = get_env("HOME")
SECRET_KEY = get_env("BURL_SECRET_KEY")

HASHID_ALPHABET = get_env(
    "BURL_HASHID_ALPHABET", "abcdefghjkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ0123456789"
)
BURL_BLACKLIST = ["admin", "api", "static", "media"]
ROUGH_COUNT_MIN = 1000

ROOT_URLCONF = "django_burl.urls"

DEFAULT_REDIRECT_URL = get_env(
    "DEFAULT_REDIRECT_URL", "https://en.wikipedia.org/wiki/Main_Page"
)

# Application definition

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "rest_framework.authtoken",
    "django.contrib.admin",
    "django.contrib.sites",
    "django_filters",
    "django_burl",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.contrib.sites.middleware.CurrentSiteMiddleware",
]


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "NAME": get_env("BURL_POSTGRES_DB", "burl"),
        "USER": get_env("BURL_POSTGRES_USER", "burl"),
        "PASSWORD": get_env("BURL_POSTGRES_PASSWORD", "burl"),
        "HOST": get_env("BURL_POSTGRES_HOST", "127.0.0.1"),
        "PORT": int(get_env("BURL_POSTGRES_PORT", 5432)),
    },
}


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = "en-us"

TIMEZONE = get_env("BURL_TIMEZONE", "America/Los_Angeles")

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_URL = "/static/"


LOG_DIR = get_env("BURL_LOG_DIR", PROJECT_ROOT)

BURL_LOG_LEVEL = get_env("BURL_LOG_LEVEL", "WARNING")
APP_LOG_LEVEL = get_env("BURL_APP_LOG_LEVEL", "INFO")


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
        "simple": {"format": "%(levelname)s %(message)s"},
    },
    "handlers": {
        "file": {
            "level": BURL_LOG_LEVEL,
            "class": "logging.handlers.RotatingFileHandler",
            "filename": os.path.join(LOG_DIR, "burl.log"),
            "maxBytes": 1024 * 1024 * 5,  # 5MiB
            "backupCount": 5,
            "formatter": "verbose",
        },
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "verbose",
            "level": BURL_LOG_LEVEL,
        },
    },
    "loggers": {
        "": {
            "handlers": ["console"],
            "level": APP_LOG_LEVEL,
        },
        "burl": {
            "handlers": ["console"],
            "level": BURL_LOG_LEVEL,
        },
    },
}

API_PAGE_SIZE = int(get_env("BURL_API_PAGE_SIZE", 20))

REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.LimitOffsetPagination",
    "PAGE_SIZE": API_PAGE_SIZE,
    "DEFAULT_PARSER_CLASSES": [
        "rest_framework.parsers.JSONParser",
    ],
    "DEFAULT_FILTER_BACKENDS": ["django_filters.rest_framework.DjangoFilterBackend"],
}

CORS_ORIGIN_ALLOW_ALL = True
CORS_ALLOW_CREDENTIALS = True

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
